"use strict";

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { "default": obj }; };

var _objectWithoutProperties = function (obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = makeAsyncScript;

var _React$PropTypes = require("react");

var _React$PropTypes2 = _interopRequireWildcard(_React$PropTypes);

"use strict";

var SCRIPT_MAP = {};

// A counter used to generate a unique id for each component that uses
// ScriptLoaderMixin.
var idCount = 0;

function makeAsyncScript(Component, scriptURL, options) {
  var AsyncScriptLoader = _React$PropTypes2["default"].createClass({
    displayName: "AsyncScriptLoader",

    propTypes: {
      onLoad: _React$PropTypes.PropTypes.func },

    statics: {
      triggerOnScriptLoaded: function triggerOnScriptLoaded() {
        var mapEntry = SCRIPT_MAP[scriptURL];
        if (!mapEntry || !mapEntry.loaded) {
          throw new Error("Script is not loaded.");
        }
        Object.keys(mapEntry.observers).forEach(function (obsKey) {
          mapEntry.observers[obsKey](mapEntry);
        });
        delete window[options.callbackName];
      } },

    getInitialState: function getInitialState() {
      return {};
    },

    getScriptLoaderID: function getScriptLoaderID() {
      if (!this.__scriptLoaderID) {
        this.__scriptLoaderID = "async-script-loader-" + idCount++;
      }
      return this.__scriptLoaderID;
    },

    getComponent: function getComponent() {
      return this.refs.childComponent;
    },

    componentDidMount: function componentDidMount() {
      var key = this.getScriptLoaderID();
      var globalName = options.globalName;
      var callbackName = options.callbackName;

      if (globalName && typeof window[globalName] !== "undefined") {
        SCRIPT_MAP[scriptURL] = { loaded: true };
      }

      if (SCRIPT_MAP[scriptURL]) {
        var entry = SCRIPT_MAP[scriptURL];
        if (entry.loaded || entry.errored) {
          this.handleLoad(entry);
          return;
        }
        entry.observers[key] = this.handleLoad;
        return;
      }

      var observers = new Map();
      observers[key] = this.handleLoad;
      SCRIPT_MAP[scriptURL] = {
        loaded: false,
        observers: observers };

      var script = document.createElement("script");

      script.src = scriptURL;
      script.async = 1;

      var callObserverFuncAndRemoveObserver = function callObserverFuncAndRemoveObserver(func) {
        var mapEntry = SCRIPT_MAP[scriptURL];
        var observersMap = mapEntry ? mapEntry.observers : {};

        Object.keys(observersMap).forEach(function (obsKey) {
          var observer = observersMap[obsKey];
          if (func(observer)) {
            delete observersMap[obsKey];
          }
        });
      };

      if (callbackName) {
        window[callbackName] = AsyncScriptLoader.triggerOnScriptLoaded;
      }

      script.onload = function () {
        var mapEntry = SCRIPT_MAP[scriptURL];
        mapEntry.loaded = true;
        callObserverFuncAndRemoveObserver(function (observer) {
          if (callbackName) {
            return false;
          }
          observer(mapEntry);
          return true;
        });
      };
      script.onerror = function (event) {
        var mapEntry = SCRIPT_MAP[scriptURL];
        mapEntry.errored = true;
        callObserverFuncAndRemoveObserver(function (observer) {
          observer(mapEntry);
          return true;
        });
      };

      // (old) MSIE browsers may call "onreadystatechange" instead of "onload"
      script.onreadystatechange = function () {
        if (this.readyState === "loaded") {
          // wait for other events, then call onload if default onload hadn't been called
          window.setTimeout(function () {
            if (SCRIPT_MAP[scriptURL].loaded !== true) {
              script.onload();
            }
          }, 0);
        }
      };

      document.body.appendChild(script);
    },

    handleLoad: function handleLoad(state) {
      this.setState(state, this.props.onLoad);
    },

    componentWillUnmount: function componentWillUnmount() {
      // Clean the observer entry
      var mapEntry = SCRIPT_MAP[scriptURL];
      if (mapEntry) {
        delete mapEntry.observers[this.getScriptLoaderID()];
      }
    },

    render: function render() {
      var globalName = options.globalName;
      var _props = this.props;
      var onLoad = _props.onLoad;

      var childProps = _objectWithoutProperties(_props, ["onLoad"]);

      if (globalName && typeof window !== "undefined") {
        childProps[globalName] = typeof window[globalName] !== "undefined" ? window[globalName] : undefined;
      }
      return _React$PropTypes2["default"].createElement(Component, _extends({ ref: "childComponent" }, childProps));
    } });

  return AsyncScriptLoader;
}

;
module.exports = exports["default"];